#include <misc.h>
#include <params.h>

module comspe

!----------------------------------------------------------------------- 
! 
! Purpose: Spectral space arrays
! 
! Method: 
! 
! Author: CCM Core Group
! $Author: pworley $
! $Id: comspe.F90,v 1.2.6.2 2002/10/28 01:12:04 pworley Exp $
! 
!-----------------------------------------------------------------------

  use shr_kind_mod, only: r8 => shr_kind_r8
  use infnan
  use pmgrid, only: plev, plat
  use pspect

  implicit none
!
! $Id: comspe.F90,v 1.2.6.2 2002/10/28 01:12:04 pworley Exp $
! $Author: pworley $
!
! Spectral space arrays
!
  real(r8) :: vz(psp,plev) = inf      ! Vorticity spectral coefficients
  real(r8) :: d(psp,plev)  = inf      ! Divergence spectral coefficients
  real(r8) :: t(psp,plev)  = inf      ! Temperature spectral coefficients
  real(r8) :: alps(psp)    = inf      ! Log-pressure spectral coefficients

  integer :: ncutoff       = bigint   ! Break-even point for vector lengths in GRCALC
  integer :: nalp(pmax)    = bigint   ! Pointer into polynomial arrays
#if ( defined SPMD )
  integer :: numm(0:plat-1) = bigint  ! number of Fourier wavenumbers owned per task
  integer :: maxm           = bigint  ! max number of Fourier wavenumbers per MPI task
  integer, dimension(:,:), allocatable :: locm ! assignment of wavenumbers to MPI tasks
#else
  integer :: numm(0:0)      = pmmax
  integer :: maxm           = pmmax
  integer, dimension(:,:), allocatable :: locm ! wavenumber ordering
#endif

#if ( defined PVP )
  integer :: ncoefi(pmaxp) = bigint   ! Pointer to start of coefficient diagonals
  integer :: nm(pmax)      = bigint   ! Number of coeffs stored on a given diagonal
  integer :: nco2(pmax)    = bigint   ! Complex form of ncoefi
#else
  integer :: nstart(pmmax) = bigint   ! Starting indices for spectral arrays (real)
  integer :: nlen(pmmax)   = bigint   ! Length vectors for spectral arrays
#endif
  real(r8) :: alp(pspt,plat/2)  = inf ! Legendre polynomials
  real(r8) :: dalp(pspt,plat/2) = inf ! Legendre polynomial derivatives

end module comspe
