#include <misc.h>
#include <params.h>

logical function prnpnt (flat, flon, iunit)
!----------------------------------------------------------------------
! Slab Ocean Model (SOM) diagnostic printout
!
! Determine if input longitude,latitude pair is among
! selected for printout; if so, returns true and unit (iunit)
! 
!---------------------------Code history--------------------------------
!
! Original version:  B. Briegleb  April 1995
! Reviewed:          B. Briegleb  March 1996
! Reviewed:          J. Kiehl     April 1996
! Reviewed:          B. Briegleb  May   1996
!
!------------------------------------------------------------------------
!
! $Id: prnpnt.F90,v 1.2.2.1 2002/06/15 13:49:07 erik Exp $
! $Author: erik $
!
!------------------------------------------------------------------------
   use shr_kind_mod, only: r8 => shr_kind_r8

   implicit none

   integer, parameter :: npnt = 18      ! total number of points to be printed
!
! Input arguments
!
   real(r8)   flat   ! latitude of point  (-90 to +90)
   real(r8)   flon   ! longitude of point (-180 to +180)
!
! Output argument
!
   integer iunit ! unit for print of point
!
! Local work space
!
   integer      n   ! index for search
!
   real(r8)         tol     ! latitude and longitude tolerance in degrees
   real(r8) latpnt(npnt)    ! latitude of desired points
   real(r8) lonpnt(npnt)    ! longitude of desired points
!
   integer untout(npnt) ! unit of output points
!
   data latpnt / 88.,  70.,  60., -65.,  -65., -75.,    1.0, &
                 60.,  75., -66., -70.,  -50., 66.1, -76.74, &
                 55.,  48.,  74., -63./
   data lonpnt /   0.,-170., -85., -170., -40., -170.,  150., &
                -30.9, 120.,  70., -110.,  90., -28.1,168.75, &
               -170.0, 170., 116.,  -64./
   data tol    /  1.5 /
   data untout /  81,   82,   83,   84,    85,   86  , 87 , &
                  88,   89,   90,   91,    92,   93  , 94 , &
                  95,   96,   97,   98/
!
!------------------------------------------------------------------------
!
! Search to see if desire to select this point:
!
   prnpnt = .false.
   do n=1,npnt
      if (abs (flat - latpnt(n)) < tol) then
         if (abs(flon-lonpnt(n)) < tol) then
            prnpnt  = .true.
            iunit   = untout(n)
         endif
      endif
   end do
!
   return
end function prnpnt
  
