module cldcond

!-----------------------------------------------------------------------------------
! Purpose: CAM interface to the cloud condensate parameterization of 
!          Rasch and Kristjansson (1998, J. Climate).
!
! Author:  Brian Eaton
!-----------------------------------------------------------------------------------

   use shr_kind_mod, only: r8=>shr_kind_r8
   use ppgrid,       only: pver, pcols, begchunk, endchunk

   implicit none
   private
   save

! Public methods

   public ::&
      cldcond_register       ! register physics buffer fields and constituent

! Private module data

   integer ::& ! indices for fields in the physics buffer
      qcwat_idx,    &
      tcwat_idx,    &
      cld_idx,      &
      lcwat_idx

!=========================================================================================
contains
!=========================================================================================

subroutine cldcond_register()

! Register physics buffer fields and constituents

   use phys_buffer, only: pbuf_times, pbuf_add

   implicit none
!------------------------------------------------------------

! Request physics buffer space for fields that persist across timesteps.

   call pbuf_add('QCWAT', 'global', 1,pver,pbuf_times, qcwat_idx)  ! 
   call pbuf_add('TCWAT', 'global', 1,pver,pbuf_times, tcwat_idx)  ! 
   call pbuf_add('CLD',   'global', 1,pver,pbuf_times, cld_idx)  ! 
   call pbuf_add('LCWAT', 'global', 1,pver,pbuf_times, lcwat_idx)  ! 

end subroutine cldcond_register

!=========================================================================================

end module cldcond
