subroutine initindx
!----------------------------------------------------------------------- 
! 
! Purpose: Registers constituents and determines index values.
! 
! Author:    CSM Contact: M. Vertenstein, Aug. 1997
!            B.A. Boville, Oct 2001
! 
!-----------------------------------------------------------------------
  use shr_kind_mod, only: r8 => shr_kind_r8
  use constituents, only: pcnst, ppcnst, cnst_add, advected, nonadvec, cnst_chk_dim, cnst_name
  use phys_buffer,  only: pbuf_init
  use chemistry,    only: trace_gas, chem_register
  use physconst,    only: mwdry, cpair, mwh2o, cph2o
  use test_tracers, only: trace_test1, trace_test2, trace_test3, test_tracers_register
  use constituents, only: ixcldw, dcconnam, sflxnam, hadvnam, vadvnam, fixcnam, tendnam, tottnam
  use cldcond,      only: cldcond_register

  implicit none
!-----------------------------------------------------------------------
#include <comctl.h>
!---------------------------Local variables-----------------------------
!
  integer m            ! loop index
  integer mm           ! constituent index 

  logical, parameter :: cldw_adv=.false.  ! true => cloud water is treated as advected tracer
!-----------------------------------------------------------------------
! Initialize physics buffer
  call pbuf_init()

!-----------------------------------------------------------------------
! Register constituents and tracers and set starting indexes
! Note that public starting indexes should not be required, these should be 
! private to the package (module) implementing the tracer and may be obtained
! by calling cnst_get_ind
!-----------------------------------------------------------------------
!
! Register water vapor
  call cnst_add('Q', advected, mwh2o, cph2o, 1.E-12_r8, mm, &
                longname='Specific humidity', readiv=.true.)
!
! Register cloud water and determine index (either advected or non-adv).
  if (cldw_adv) then
     call cnst_add('CWAT', advected, mwdry, cpair, 0._r8, ixcldw, &
                    longname='Total Grid box averaged Condensate Amount (liquid + ice)')
  else
     call cnst_add('CWAT', nonadvec, mwdry, cpair, 0._r8, ixcldw, &
                    longname='Total Grid box averaged Condensate Amount (liquid + ice)')
  endif

  call cldcond_register()
!
! Register chemical constituents
  if (trace_gas) then
     call chem_register()
  endif
!
! Register advected test tracers and determine starting index
  if (trace_test1 .or. trace_test2 .or. trace_test3) then
     call test_tracers_register()
  end if
!
! All tracers registered, check that the dimensions are correct
  call cnst_chk_dim()
!
! Set default names for non-water advected and non-advected tracers
! Set names of advected and non-advected tracer diagnostics
!
  do m=1,ppcnst
     dcconnam(m) = 'DC'//cnst_name(m)
     sflxnam(m)  = 'SF'//cnst_name(m)
  end do
  do m=1,pcnst
     hadvnam(m)  = 'HA'//cnst_name(m)
     vadvnam(m)  = 'VA'//cnst_name(m)
     fixcnam(m)  = 'DF'//cnst_name(m)
     tendnam(m)  = 'TE'//cnst_name(m)
     tottnam(m)  = 'TA'//cnst_name(m)
  end do

end subroutine initindx
