module include

  use shr_kind_mod, only: r8 => shr_kind_r8
  implicit none

  ! Set parameters for the floating point flags "nan" (not-a-number).
  ! and "bigint" the point at which integers start to overflow. 
  ! These values are used to initialize arrays and to detect if arrays
  ! are being used before being set.

  real(r8), parameter :: nan = O'0777700000000000000000'
  integer,  parameter :: bigint = O'17777777777'    ! largest possible 32-bit integer

  ! Variables read in from surface file

  integer :: lsmlon         !number of grid longitudes
  integer :: lsmlat         !number of grid latitudes
  integer :: nlevsoi        !number of soil levels
  integer :: nlevlak        !number of lake levels
  integer :: maxpatch_pft   !maximum number of PFT subgrid patches per grid cell
  integer :: maxpatch       !maximum number of subgrid patches per grid cell
  logical :: do_DGVM        !true => do DGVM

  integer, allocatable :: numlon(:)     !longitude points for each latitude strip
  integer, allocatable :: landmask(:,:) !land mask: 1 = land. 0 = ocean

  ! Variables derived from surface file

  integer :: nland     ! clm2.0 - number of grid cells points 
  integer :: npatch    ! clm2.0 - total number of patches 

  integer :: numgrid   ! clm2.1 - number of gridcells 
  integer :: numland   ! clm2.1 - number of landunits 
  integer :: numcols   ! clm2.1 - number of columns 
  integer :: numpfts   ! clm2.1 - number of pfts 

  ! Number of plant types

  integer, parameter :: numpft = 16   !number of plant types

  ! Number of snow levels

  integer, parameter :: nlevsno = 5   !maximum number of snow layers


  ! RTM river routing model

  integer, parameter :: rtmlon = 720  !# of rtm longitudes
  integer, parameter :: rtmlat = 360  !# of rtm latitudes
  real(r8) :: volr(rtmlon,rtmlat)     !water volume in cell (m^3)

  ! 1d mapping - clm2.0

  type land1d
     integer ,pointer :: patch(:,:)  !patch vector index (nland x maxpatch)
     real(r8),pointer :: wtxy(:,:)   !subgrid weights (nland x maxpatch)
  end type land1d

  type subgrid1d
     integer , pointer :: mxy(:)     !subgrid patch type index (npatch)
     real(r8), pointer :: wtxy(:)    !subgrid weight for each patch point (npatch)
  end type subgrid1d

  type (land1d)    :: landvec
  type (subgrid1d) :: patchvec

  real(r8), parameter :: spval = 1.e36  !special value for missing data (ocean)

end module include

